# -*- coding: utf-8 -*-
# Copyright (C) 2007-2008 www.stani.be
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/
import math
from Blender import Image, Material, Object, Scene, Texture

OBJECTS = ['Box', 'CD']

def color(hex_color):
    # converts color str (#<three hex pairs>) to list of three floats
    # [0.0, 1.0] and returns it
    return [int(hex_color[1:][i:i+2], 16)/255.0
            for i in xrange(0, len(hex_color)-1, 2)]


def set_up_render(args):

    def boolean(arg):
        return args[arg].lower() in ('yes','true')

    scene = Scene.GetCurrent()
    #which object?
    obj_name = args['object']
    #layer setting (layer 1 = box, 2 = cd, etc.). try 2 just for fun :)
    scene.setLayers([OBJECTS.index(obj_name)+1])
    obj = Object.Get(obj_name)

    #object rotation
    obj.RotX = math.radians(float(args['rotation_x']))
    obj.RotY = -math.radians(float(args['rotation_y']))
    obj.RotZ = math.radians(float(args['rotation_z']))

    #load image as texture
    texture = Texture.Get('front_tex')
    image = Image.Load(args['input_image'])
    texture.image = image

    #world
    context = scene.getRenderingContext()
    if boolean('alpha'):
        context.alphaMode = 1
        context.enableRGBAColor()
    else:
        world = scene.world
        world.setZen(color(args['gradient_top']))
        world.setHor(color(args['gradient_bottom']))
        #stars
        mode = world.getMode()
        if boolean('stars'):
            mode |= 2
            world.setMode(mode)
            #set stars color
            stars_color = color(args['stars_color'])
            old_options = world.getStar()
            new_options = stars_color + old_options[3:]
            world.setStar(new_options)
        #mist
        if boolean('mist'):
            mode |= 1
            world.setMode(mode)

    #floor
    if boolean('floor'):
        #set floor color
        floor_material = Material.Get('floor')
        floor_material.setRGBCol(color(args['floor_color']))
        #set floor reflection
        ref = int(args['floor_reflection'])
        if ref:
            floor_material.setFresnelMirrFac(2-ref/100.0)
        else:
            mode = floor_material.getMode()
            floor_material.setMode(mode ^ Material.Modes['RAYMIRROR'])
    else:
        #remove floor
        scene.objects.unlink(Object.Get('floor'))

    #specific object functions
    if obj_name == 'Box':
        #box color
        box_material = Material.Get('box')
        box_material.setRGBCol(color(args['box_color']))
        #adapt proportions of box
        width, height = image.getSize()
        fac = height/0.8
        box_width = width/fac
        box_height = height/fac
        box_depth = float(args['box_depth'])/fac
        obj.setSize(box_width, box_depth, box_height)

