# Phatch - Photo Batch Processor
# Copyright (C) 2007-2009 www.stani.be
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see http://www.gnu.org/licenses/
#
# Phatch recommends SPE (http://pythonide.stani.be) for editing python files.

# Follows PEP8

"""All PIL related issues."""

#FIXME:
# - info should be defined on layer level
#   -> move call afterwards also to layer level
#   -> adapt image inspector

try:
    _
except NameError:
    _ = unicode

import datetime
import os
import operator
import re
import tempfile
import time
import types

import Image

#todo make this lazy
from translation import _t
from lib import formField
from lib import imtools
from lib import system
from lib.desktop import DESKTOP_FOLDER
from other import EXIF

try:
    from ExifTags import TAGS as EXIFTAGS
except:
    #older versions of PIL
    EXIFTAGS = {}

try:
    import pyexiv2
    from lib import _pyexiv2 as exif
except:
    pyexiv2 = None
    exif = False
WWW_PYEXIV2 = 'http://tilloy.net/dev/pyexiv2/'
NEEDS_PYEXIV2 = _('pyexiv2 needs to be installed')+' (%s)'%WWW_PYEXIV2

try:
    from lib.unicoding import ensure_unicode
    from ct import TITLE
    from translation import _t
    from config import USER_BIN_PATH
    from lib.formField import RE_FILE_IN, RE_FILE_OUT
    from lib import system

except ImportError:

    def ensure_unicode(x):
        return x

    TITLE = ''
    _t = str
    USER_BIN_PATH = os.path.expanduser('~/.phatch/bin')
    system.is_www_file = os.path.isfile #fix this

from lib import system
from lib import openImage
system.set_bin_paths([USER_BIN_PATH])


EXT_BY_FORMATS = {
    'GIF': ['gif'],
    'JPEG': ['jpg', 'jpeg', 'jpe'],
    'PNG': ['png'],
    'TIFF': ['tif', 'tiff'],
}
FORMATS_BY_EXT = {}
for format, exts in EXT_BY_FORMATS.items():
    for ext in exts:
        FORMATS_BY_EXT[ext] = format
IMAGE_DEFAULT_DPI = 72
DOT = '.' #should be same as in core.translations
MONTHS = (_t('January'), _t('February'), _t('March'), _t('April'),
    _t('May'), _t('June'), _t('July'), _t('August'), _t('September'),
    _t('October'), _t('November'), _t('December'))
WEEKDAYS = (_t('Monday'), _t('Tuesday'), _t('Wednesday'), _t('Thursday'),
    _t('Friday'), _t('Saturday'), _t('Sunday'))
DATETIME_KEYS = ['year', 'month', 'day', 'hour', 'minute', 'second']
re_DATETIME = re.compile(
                '(?P<year>\d{4})[-:](?P<month>\d{2})[-:](?P<day>\d{2}) '
                '(?P<hour>\d{2}):(?P<minute>\d{2}):(?P<second>\d{2})')

re_TAG = re.compile('(Pil|Exif|Iptc|Pexif|Zexif)([.]\w+)+')
re_KEY = re.compile('(#*)((\w|[.])*$|[$])')
THUMB_SIZE = (128, 128)
WWW_CACHE = {}

IMAGE_READ_EXTENSIONS = formField.IMAGE_READ_EXTENSIONS + \
    openImage.WITHOUT_PIL.extensions
IMAGE_READ_EXTENSIONS.sort()

IMAGE_EXTENSIONS = [ext for ext in IMAGE_READ_EXTENSIONS
    if ext in formField.IMAGE_WRITE_EXTENSIONS]


def check_libtiff(compression):
    if not(openImage.open_libtiff or compression in ['raw', 'None']):
        raise Exception('Libtiff tools are needed for "%s" compression'\
            %compression)


def file_extension(uri):
    return os.path.splitext(uri)[-1][1:].lower()


def open_image_with_pil(uri):
    image = imtools.open_image(uri)
    #types which PIL can open, but not load
    compression = image.info.get('compression', 'none')
    if hasattr(compression, 'startswith') and \
            compression.startswith('group'):
        # tiff image with group4 compression
        check_libtiff('g4') #raise exception if openImage not present
        image = openImage.open_libtiff(uri)
    return image


def open_image_without_pil(filename, method_register):
    """Try to open images which PIL can't handle."""
    extension = file_extension(filename)
    if extension in method_register.extensions:
        methods = method_register.get_methods(extension)
        for open_method in methods:
            image = open_method(filename)
            if image:
                return image


def open_image(uri):
    local = not system.is_www_file(uri)
    # svg, pdf, ...
    if local:
        image = open_image_without_pil(uri, openImage.WITHOUT_PIL)
    if image:
        return image
    # pil
    try:
        return open_image_with_pil(uri)
    except IOError, message:
        pass
    # tiff (which pil can only handle partly)
    if local:
        image = open_image_without_pil(uri, openImage.ENHANCE_PIL)
    else:
        image = None
    if image is None:
        raise IOError(message)
    return imtools.open_image(uri)


def get_format(ext):
    return FORMATS_BY_EXT.get(ext.lower(), ext.upper())


def transpose(image, methods):
    """Transpose with a sequence of transformations, mainly useful
    for exif."""
    for method in methods:
        image = image.transpose(method)
    return image


def extract_info_dpi(info, image):
    #info
    image_info = image.info
    if 'dpi' in image_info:
        #also in this case it is possible that dpi is 0
        dpi = image_info['dpi']
        if len(dpi) > 1:
            dpi = dpi[0]
        info[_t('dpi')] = dpi
    else:
        dpi = 0
    if dpi == 0:
        if 'aspect' in image_info:
            info[_t('dpi')] = dpi = image_info['aspect'][0]
        else:
            dpi = 0
        if dpi == 0:
            info[_t('dpi')] = IMAGE_DEFAULT_DPI


def split_data(new):
    """new - dictionary"""
    value = new.values()[0]
    #tuples or list
    if type(value) in (types.ListType, types.TupleType):
        if len(value) > 1:
            for k, v in new.items():
                for i, x in enumerate(v):
                    new['%s%s%d'%(k, DOT, i)] = v[i]
        return
    #datetime strings
    done = False
    for k, v in new.items():
        if type(v) in types.StringTypes:
            dt = re_DATETIME.match(v)
            if dt:
                for key in DATETIME_KEYS:
                    new['%s%s%s'%(k, DOT, key)] = dt.group(key)
                    done = True
    if done:
        return
    #date time values
    if type(value) == datetime.datetime:
        for k, v in new.items():
            for key in DATETIME_KEYS:
                new['%s%s%s'%(k, DOT, key)] = getattr(v, key)


def extract_info_pexif(info, image):
    """pexif = Pil EXIF"""
    #check if exif information is available through pil
    if not hasattr(image, '_getexif'):
        return info
    #add exif information
    try:
        exif = image._getexif()
    except:
        exif = None
    if not exif:
        return
    for key, value in exif.items():
        new = {'Zexif%s0x%04x'%(DOT, key): value}
        if key in EXIFTAGS:
            new['Pexif%s%s'%(DOT, EXIFTAGS[key])] = value
        #Check if we can split exif data further
        split_data(new)
        info.update(new)


def extract_info_pyexiv2(info, path=''):
    if not (pyexiv2 and os.path.isfile(path)):
        info['exif'] = []
        info['iptc'] = []
        return
    #official patch of pyexiv2
    try:
        if type(path) is unicode:
            path = path.encode('utf-8')
    except UnicodeEncodeError:
        return
    try:
        image = pyexiv2.Image(path)
        image.readMetadata()
    except:
        return
    #exif & iptc
    exif_keys = info['exif'] = image.exifKeys()
    iptc_keys = info['iptc'] = image.iptcKeys()
    for keys in [exif_keys, iptc_keys]:
        for key in keys:
            try:
                new = {key: image[key]}
                split_data(new)
                info.update(new)
            except:
                pass
    return image


def fix_EXIF(tag):
    if not tag.startswith('EXIF'):
        tag = 'EXIF.'+tag
    return tag.replace(' ', DOT)


def extract_info_EXIF(info, path=''):
    if not os.path.isfile(path):
        return
    f = open(path, 'rb')
    try:
        tags = EXIF.process_file(f)
    except:
        tags = {}
    f.close()
    for tag, value in tags.items():
        if tag != 'EXIF UserComment':
            info[fix_EXIF(tag)] = value


def extract_info_path(info, path='', folder=None):
    if not path.strip():
        return
    path = ensure_unicode(path)
    dirname = os.path.dirname(path)
    if folder:
        info[_t('folder')] = folder
    else:
        info[_t('folder')] = dirname
    info[_t('root')], \
    info[_t('foldername')] = os.path.split(info[_t('folder')])
    info[_t('subfolder')] = dirname[len(info[_t('folder')])+1:]
    info[_t('path')] = path
    filename = os.path.basename(path)
    info[_t('filename')], typ = os.path.splitext(filename)
    info[_t('type')] = typ[1:]
    #os time
    try:
        file_stat = os.stat(path)
        info[_t('filesize')] = file_stat.st_size
        st_mtime = time.localtime(file_stat.st_mtime)[:7]
    except:
        info[_t('filesize')] = 0
        st_mtime = (0, )*7
    info[_t('year')], info[_t('month')], info[_t('day')], \
        info[_t('hour')], info[_t('minute')], info[_t('second')], \
        info[_t('weekday')] = st_mtime
    info[_t('weekdayname')] = WEEKDAYS[info[_t('weekday')]]
    info[_t('monthname')] = MONTHS[info[_t('month')]-1]


def extract_info_image(info, image):
    extract_info_dpi(info, image)
    info['compression'] = image.info.get('compression', 'none')
    info['transparency'] = image.info.get('transparency', None)
    #load Pil info
    for key, value in image.info.items():
        if key!='exif':
            info['Pil'+DOT+key.title()] = value
    #summarize all photo properties in info
    info.update({
        'Pil'+DOT+'Format': image.format,
        'Pil'+DOT+'FormatDescription': image.format_description,
    })
    return info


def extract_info_geometry(info, image):
    #retrieve properties
    width, height = size = image.size
    info.update({
        _t('width'): width,
        _t('height'): height,
        _t('mode'): image.mode,
        'Pil'+DOT+'Size': size,
    })
    return info


def extract_info_pil(image):
    return extract_info_image({}, image)
    #20090705 Nadia: prefix_dict is taking 16.22% of the total execution time
    #return prefix_dict(extract_info_image({}, image), 'new'+DOT)

def extract_info(info, image, path='', image_index=0,
        format=None, metadata=False):
    info['#'] = image_index
    if not('root' in info.keys()):
        extract_info_path(info, path)
    extract_info_image(info, image)
    if format:
        info['Pil.Format'] = format
    if metadata:
        pyexiv2_image = extract_info_metadata(info, path, image)
    else:
        pyexiv2_image = None
    return pyexiv2_image


def extract_info_metadata(info, path, image=None):
    if image:
        extract_info_pexif(info, image)
    pyexiv2_image = extract_info_pyexiv2(info, path)
    extract_info_EXIF(info, path)
    return pyexiv2_image


def get_exif_transposition(image):
    orientation = get_exif_orientation(image)
    #see EXIF.py
    if orientation == 1:
        transposition = transposition_reverse = ()
    elif orientation == 2:
        transposition = Image.FLIP_LEFT_RIGHT,
        transposition_reverse = Image.FLIP_LEFT_RIGHT,
    elif orientation == 3:
        transposition = Image.ROTATE_180,
        transposition_reverse = Image.ROTATE_180,
    elif orientation == 4:
        transposition = Image.FLIP_TOP_BOTTOM,
        transposition_reverse = Image.FLIP_TOP_BOTTOM,
    elif orientation == 5:
        transposition = Image.FLIP_LEFT_RIGHT, \
                                        Image.ROTATE_90
        transposition_reverse = Image.ROTATE_270, \
                                        Image.FLIP_LEFT_RIGHT
    elif orientation == 6:
        transposition = Image.ROTATE_270,
        transposition_reverse = Image.ROTATE_90,
    elif orientation == 7:
        transposition = Image.FLIP_LEFT_RIGHT, \
                                        Image.ROTATE_270
        transposition_reverse = Image.ROTATE_90, \
                                        Image.FLIP_LEFT_RIGHT
    elif orientation == 8:
        transposition = Image.ROTATE_90,
        transposition_reverse = Image.ROTATE_270,
    else:
        transposition = transposition_reverse = ()
    return transposition, transposition_reverse


def exif_transpose(image):
    return transpose(image, get_exif_transposition(image)[0])


def get_exif_orientation(image):
    if not hasattr(image, '_getexif'):
        return 1
    try:
        _exif = image._getexif()
        if not _exif:
            return 1
        return _exif[0x0112]
    except KeyError:
        return 1


def prefix_dict(d, prefix, exclude=[]):
    exclude.append(prefix)
    for key, value in d.items():
        add = True
        for e in exclude:
            if key[:len(e)] == e:
                add = False
                break
        if add:
            d[prefix+key] = value
    return d


class TestInfo:

    def __init__(self, info):
        self._keys = info.keys()
        self._keys.append(_t('desktop'))

    def __getitem__(self, x):
        key_match = re_KEY.match(x)
        if key_match:
            key = key_match.group(2)
            if not key:
                key = '#'
        else:
            key = x
        if self._is_valid_key(key):
            return '2'
        raise KeyError(x)

    def _is_valid_key(self, x):
        return x in self._keys or re_TAG.match(x)


class Info:

    def __init__(self, dictionary=None, base=0):
        if dictionary is None:
            self._dictionary= {'#': 0}
        else:
            self._dictionary= dictionary
        self._dictionary[_t('desktop')] = DESKTOP_FOLDER
        self.base = 0
        for attr in ['__setitem__', '__delitem__', 'keys']:
            setattr(self, attr, getattr(self._dictionary, attr))
        self.get = self._dictionary.get
        self.keys = self._dictionary.keys
        self.items = self._dictionary.items
        self.update = self._dictionary.update
        self.values = self._dictionary.values

    def __getitem__(self, x):
        match = re_KEY.match(x)
        if match:
            number = match.group(1)
            if number:
                key = match.group(2)
                if not key:
                    key = '#'
                format = '%%0%dd'%len(number)
                value = self._dictionary[key]
                if key[-1] in ('#', '$'):
                    value += self.base
                return format%value
        return self._dictionary[x]

    def copy(self):
        return Info(self._dictionary.copy())


def create_dummy_info():
    global TEST_INFO
    image = Image.new('L', (1, 1))
    info = {}
    pyexiv2_image = extract_info(info, image, '/test/unknown.png')
    info = extract_info_geometry(info, image)
    TEST_INFO = TestInfo(info)
    return TEST_INFO


def image_to_dict(filename, im=None):
    folder, name = os.path.split(filename)
    d = {'path': filename, 'filename': name}
    if im:
        width, height = im.size
        d['width'] = width
        d['height'] = height
        d['mode'] = im.mode
    return d


def verify_image(image_info, valid, invalid):
    try:
        im = Layer(image_info['path'], load=False).image
        #if info has 'open', the image is not opened by PIL
        #and already loaded and verified
        if not ('open' in im.info):
            im.verify()
        #extract_info_geometry(image_info, im)
        valid.append(image_info)
    except Exception, error:
        invalid.append(image_info['path'])


class Photo:

    def __init__(self, filename, image_index=0, image_info=None):
        #validate image
        name = self.current_layer_name = _t('background')
        layer = Layer(filename, load=True)
        self.layers = {name: layer}
        self.log = ''
        self.metadata = {}
        self.modify_date = None
        self.report_files = [] # for reports
        self.source_file = filename
        self._exif_transposition_orientation = None
        self._exif_transposition_reverse= None
        self.init_info(layer, filename, image_index, image_info)

    def flush_metadata(self):
        #is there something to be flushed?
        if not self.metadata:
            return
        #throw an error if pyexiv2 is not installed
        if not exif:
            raise ImportError(NEEDS_PYEXIV2)
        path = self.info['path']
        self.log += exif.flush(path, self.metadata,
            self.info['Pil.Format'])
        self.update_exif_source()

    def update_exif_source(self, save_metadata=True, info=None):
        if info is None:
            info = self.info
        format = info['Pil.Format']
        path = info['path']
        if format:
            format = format.upper()
        if save_metadata and exif:
            self._exif_source = {'path': path, 'format': format}
        else:
            self._exif_source = None
        #as metadata has changed, use new source
        self.metadata = {}
        #update
        try:
            #pexif
            image = Image.open(path)
        except:
            image = None
        extract_info_metadata(info, path, image)

    def get_filename(self, folder, filename, typ):
        #self.info[_t('new')+DOT+_t('width')], \
        #    self.info[_t('new')+DOT+_t('height')] = \
        #    self.image.size
        return os.path.join(folder, '%s.%s'%(filename, typ))\
            .replace('<', '%(').replace('>', ')s')%self.__dict__

    def init_info(self, layer, path, image_index,
            image_info=None, prefix=_t('new')+DOT):
        self.source_info = Info(image_info)
        self.pyexiv2_image = extract_info(self.source_info, layer.image,
            path, image_index)
        #geometry info needs exif rotation first
        self.rotate_exif()
        extract_info_geometry(self.source_info, layer.image)
        self.update_exif_source(info=self.source_info)
        #fix this for multiple save
        self.info = self.source_info
        #self.info = prefix_dict(self.source_info, prefix)
    def get_info(self):
        """Use method for read-only"""
        return self.info.copy()

    def get_source_info(self):
        """Use method for read-only"""
        return self.source_info.copy()

    #---layers
    def get_flattened_image(self):
        return self.get_layer().image.copy()

    def get_layer(self, name=None):
        if name is None:
            name = self.current_layer_name
        return self.layers[name]

    def get_thumb(self, size=THUMB_SIZE):
        return imtools.thumb(self.get_flattened_image(), thumb_size=size)

    def set_layer(self, layer, name=None):
        if name is None:
            name = self.current_layer_name
        self.layers[name] = layer

    #---image operations affecting all layers
    def save(self, filename, format=None, save_metadata=True, **options):
        """Saves a flattened image"""
        #todo: flatten layers
        image = self.get_flattened_image()

        #reverse exif previously applied exif orientation
        if exif and exif.is_writable(format):
            thumb_data = imtools.get_thumb_data(image, format)
            image = transpose(image,
                self._exif_transposition_reverse)
        else:
            thumb_data = None

        #convert to rgb
        if image.mode == 'CMYK' and not (format in ['JPEG', 'TIFF']):
            self.log = \
                _('CMYK has been converted to RGB to save as %s.')\
                    %format
            image = image.convert('RGB')
        try:
            if 'compression.tif' in options:
                compression = options['compression.tif']
                del options['compression.tif']
                check_libtiff(compression)
                self.log += openImage.save_libtiff(image, filename,
                    compression=compression, **options)
            else:
                image.save(filename, **options)
                #copy metadata if needed (problematic for tiff)
                if exif and save_metadata:
                    self.copy_metadata(filename, format, TITLE,
                            metadata=save_metadata, thumb_data=thumb_data)
            if self.modify_date:
                # Update file access and modification date
                os.utime(filename, (self.modify_date, self.modify_date))
            self.append_to_report(filename, image)
        except IOError, message:
            # clean up corrupted drawing
            if os.path.exists(filename):
                os.remove(filename)
            raise IOError(message)
        #update info
        if hasattr(options, 'dpi'):
            self.set_attr(_t('dpi'), options['dpi'][0])
        extract_info(self.info, image, filename, format=format)
        self.update_exif_source(save_metadata=save_metadata)

    def append_to_report(self, filename, image=None):
        report = image_to_dict(filename, image)
        report[_t('source')] = self.source_file
        self.report_files.append(report)

    def convert(self, mode, *args, **keyw):
        """Converts all layers to a different mode."""
        for layer in self.layers.values():
            if layer.image.mode == mode:
                continue
            if mode == 'P' and imtools.has_alpha(layer.image):
                layer.image = imtools.convert(layer.image, mode, *args, **keyw)
                self.info['transparency'] = 255
            elif mode == 'P':
                layer.image = imtools.convert(layer.image, mode, *args, **keyw)
                self.info['transparency'] = None
            else:
                layer.image = imtools.convert(layer.image, mode, *args, **keyw)
        self.set_attr(_t('mode'), mode)

    def safe_mode(self, format):
        """Convert the photo into a safe mode for this specific format"""
        #TODO: Extend this helper function to support other formats as well
        image = self.get_layer().image
        if format == 'JPEG':
            if image.mode in ['RGBA', 'P']:
                self.convert('RGB')
            elif image.mode in ['LA', 'F']:
                self.convert('L')

    def resize(self, size, method):
        """Resizes all layers to a different size"""
        size = (max(1, size[0]), max(1, size[1]))
        for layer in self.layers.values():
            layer.image = layer.image.resize(size, method)
        self.set_size(size)

    def set_size(self, size):
        self.set_attr(_t('width'), size[0])
        self.set_attr(_t('height'), size[1])
        self.set_attr('Pil'+DOT+_t('Size'), size)

    def set_attr(self, attr, value):
        #self.info[_t('new')+DOT+attr] = value
        self.info[attr] = value

    def update_size(self):
        self.set_size(self.layers.values()[0].image.size)
        return self

    def rotate_exif(self, reverse=False):
        layers = self.layers.values()
        if reverse:
            transposition = self._exif_transposition_reverse
            self._exif_transposition_reverse = ()
        else:
            transposition, self._exif_transposition_reverse = \
                get_exif_transposition(layers[0].image)
        if transposition:
            for layer in layers:
                layer.image = transpose(layer.image, transposition)

    #---pil
    def apply_pil(self, function, *arg, **keyw):
        for layer in self.layers.values():
            layer.apply_pil(function, *arg, **keyw)

    #---exif
    def copy_metadata(self, filename, format=None, software=None,
            metadata=True, thumb_data=None):
        copied = False
        if self._exif_source:
            if metadata:
                update = self.metadata
            else:
                update = None
            try:
                self.log += exif.copy_metadata(
                    self._exif_source['path'],
                    filename,
                    source_format = self._exif_source['format'],
                    target_format = format,
                    reset_orientation = \
                        self._exif_transposition_orientation,
                    software = software,
                    update = update,
                    thumb_data = thumb_data,
                    pyexiv2_image = self.pyexiv2_image)
                copied = True
            except:
                pass
        if not copied:
            self._exif_transposition_orientation = 0

    #---external
    def temp_filename(self, ext):
        return tempfile.NamedTemporaryFile().name\
            +self.info['filename']+ext

    def call(self, command, check_exe=True, shell=None):
        if shell is None:
            shell = not system.WINDOWS
        #get command line
        info = self.get_info()
        layer = self.get_layer()
        image = layer.image
        #loop over input -> save to temp files
        temp_files = []
        done = []
        error = None
        for match in RE_FILE_IN.finditer(command):
            source = match.group()
            if not(source in done):
                ext = match.group(1)
                target = self.temp_filename(ext)
                try:
                    image.save(target)
                except Exception, error:
                    pass
                temp_files.append((source, target))
                done.append(source)
        #check if we have a file_in
        #clean up in case of error
        if error:
            for source, target in temp_files:
                os.remove(target)
            raise error
        #loop over output
        output = None
        for index, match in \
                enumerate(RE_FILE_OUT.finditer(command)):
            if index > 0:
                #only 1 is allowed
                raise Exception('Only one file_out.* is allowed.')
            source = match.group()
            ext = match.group(1)
            output = self.temp_filename(ext)
            command = command.replace(source, system.fix_quotes(output))
        #tweak command line
        for source, target in temp_files:
            command = command.replace(source, system.fix_quotes(target))
        #execute
        system.call(command, shell=shell)
        #give back filename
        if output and not os.path.exists(output):
            error = True
        else:
            error = False
        for source, target in temp_files:
            os.remove(target)
        if error:
            raise Exception(
                _('Command did not produce an output image:\n%s')\
                %command)
        if output:
            layer.open(output)
            extract_info_geometry(self.info, layer.image)

            try:
                os.remove(output)
            except WindowsError:
                # XXX: this fails on win7 for some reason.
                # perhaps there's still a file handle open?
                pass


class Layer:

    def __init__(self, filename, position=(0, 0), load=True):
        self.open(filename)
        self.position = position
        # VERY IMPORTANT
        # do not remove load option, otherwise openImage.py won't work
        # correctly with group4 tiff compression
        if load:
            self.image.load()

    def open(self, uri):
        self.image = open_image(uri)

    def apply_pil(self, function, *arg, **keyw):
        self.image = function(self.image, *arg, **keyw)


if __name__ == '__main__':
    create_dummy_info()
